;-----------------------------------------------------------------------------
;-      ATMEL Microcontroller Software Support  -   ROUSSET -
;-----------------------------------------------------------------------------
; The software is delivered "AS IS" without warranty or condition of any
; kind, either express, implied or statutory. This includes without
; limitation any warranty or condition with respect to merchantability or
; fitness for any particular purpose, or against the infringements of
; intellectual property rights of others.
;-----------------------------------------------------------------------------
;- File Name            : aic.inc
;- Object               : Advanced Interrupt Controller Definition File.
;- Translator           : ARM Software Development Toolkit V2.11a
;-
;- 1.0 10/03/98 JCZ     : Creation
;- 2.0 21/10/98 JCZ     : Clean up.
;-----------------------------------------------------------------------------

;-----------------------------------------
;- Advanced Interrupt Controller Structure
;-----------------------------------------
                    ^       0
AIC_SMR             #       32*4        ; Source Mode Register
AIC_SVR             #       32*4        ; Source Vector Register
AIC_IVR             #       4           ; Interrupt Vector Register
AIC_FVR             #       4           ; Fast Interrupt Vector Register
AIC_ISR             #       4           ; Interrupt Status Register
AIC_IPR             #       4           ; Interrupt Pending Register
AIC_IMR             #       4           ; Interrupt Mask Register
AIC_CISR            #       4           ; Core Interrupt Status Register
                    #       4
                    #       4
AIC_IECR            #       4           ; Interrupt Enable Command Register
AIC_IDCR            #       4           ; Interrupt Disable Command Register
AIC_ICCR            #       4           ; Interrupt Clear Command Register
AIC_ISCR            #       4           ; Interrupt Set Command Register
AIC_EOICR           #       4           ; End Of Interrupt Command Register

;---------------------------------
;- Interrupt Source Mode Registers
;---------------------------------

;- Source Type Definition
SRCTYPE             EQU     0x60
PRIOR               EQU     0x07

;----------------------------
;- Interrupt Status Register
;----------------------------
IRQID               EQU     0x1F

;---------------------------------
;- Interrupt Core Status Register
;---------------------------------
NFIQ                EQU     0x01
NIRQ                EQU     0x02

;--------------------------------
;- Device Dependancies Definition
;--------------------------------

AIC_BASE            EQU     0xFFFFF000

    IF  :DEF:AT91M40400

FIQ                 EQU     0
SWIRQ               EQU     1
US0IRQ              EQU     2
US1IRQ              EQU     3
TC0IRQ              EQU     4
TC1IRQ              EQU     5
TC2IRQ              EQU     6

WDIRQ               EQU     7
PIOIRQ              EQU     8

IRQ0                EQU     16
IRQ1                EQU     17
IRQ2                EQU     18

;- Really implemented in the 40400 not including the FIQ
NB_INTERRUPT        EQU     18

    ENDIF   ; AT91M40400

            END
